from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.v1_schema_retrieve_format import V1SchemaRetrieveFormat
from ...models.v1_schema_retrieve_lang import V1SchemaRetrieveLang
from ...models.v1_schema_retrieve_response_200 import V1SchemaRetrieveResponse200
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    format_: Union[Unset, V1SchemaRetrieveFormat] = UNSET,
    lang: Union[Unset, V1SchemaRetrieveLang] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_format_: Union[Unset, str] = UNSET
    if not isinstance(format_, Unset):
        json_format_ = format_.value

    params["format"] = json_format_

    json_lang: Union[Unset, str] = UNSET
    if not isinstance(lang, Unset):
        json_lang = lang.value

    params["lang"] = json_lang

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/v1/schema",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[V1SchemaRetrieveResponse200]:
    if response.status_code == 200:
        response_200 = V1SchemaRetrieveResponse200.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[V1SchemaRetrieveResponse200]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    format_: Union[Unset, V1SchemaRetrieveFormat] = UNSET,
    lang: Union[Unset, V1SchemaRetrieveLang] = UNSET,
) -> Response[V1SchemaRetrieveResponse200]:
    """OpenApi3 schema for this API. Format can be selected via content negotiation.

    - YAML: application/vnd.oai.openapi
    - JSON: application/vnd.oai.openapi+json

    Args:
        format_ (Union[Unset, V1SchemaRetrieveFormat]):
        lang (Union[Unset, V1SchemaRetrieveLang]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[V1SchemaRetrieveResponse200]
    """

    kwargs = _get_kwargs(
        format_=format_,
        lang=lang,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    format_: Union[Unset, V1SchemaRetrieveFormat] = UNSET,
    lang: Union[Unset, V1SchemaRetrieveLang] = UNSET,
) -> Optional[V1SchemaRetrieveResponse200]:
    """OpenApi3 schema for this API. Format can be selected via content negotiation.

    - YAML: application/vnd.oai.openapi
    - JSON: application/vnd.oai.openapi+json

    Args:
        format_ (Union[Unset, V1SchemaRetrieveFormat]):
        lang (Union[Unset, V1SchemaRetrieveLang]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        V1SchemaRetrieveResponse200
    """

    return sync_detailed(
        client=client,
        format_=format_,
        lang=lang,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    format_: Union[Unset, V1SchemaRetrieveFormat] = UNSET,
    lang: Union[Unset, V1SchemaRetrieveLang] = UNSET,
) -> Response[V1SchemaRetrieveResponse200]:
    """OpenApi3 schema for this API. Format can be selected via content negotiation.

    - YAML: application/vnd.oai.openapi
    - JSON: application/vnd.oai.openapi+json

    Args:
        format_ (Union[Unset, V1SchemaRetrieveFormat]):
        lang (Union[Unset, V1SchemaRetrieveLang]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[V1SchemaRetrieveResponse200]
    """

    kwargs = _get_kwargs(
        format_=format_,
        lang=lang,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    format_: Union[Unset, V1SchemaRetrieveFormat] = UNSET,
    lang: Union[Unset, V1SchemaRetrieveLang] = UNSET,
) -> Optional[V1SchemaRetrieveResponse200]:
    """OpenApi3 schema for this API. Format can be selected via content negotiation.

    - YAML: application/vnd.oai.openapi
    - JSON: application/vnd.oai.openapi+json

    Args:
        format_ (Union[Unset, V1SchemaRetrieveFormat]):
        lang (Union[Unset, V1SchemaRetrieveLang]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        V1SchemaRetrieveResponse200
    """

    return (
        await asyncio_detailed(
            client=client,
            format_=format_,
            lang=lang,
        )
    ).parsed



"""Contains internal utils for dealing with the client"""
import argparse
import inspect
import json
import sys
from collections import Counter
from enum import Enum
from functools import wraps
from typing import Any, Callable

import flexpolyline
import pandas as pd
from shapely import Point

from casageo.client.types import Response
from casageo.types import CasaGeoError


def load_json(response: Response[Any]) -> dict:
    data = json.loads(response.content)
    if response.status_code != 200:
        raise CasaGeoError(f"Bad status code {response.status_code}: {data}")
    return data


def load_flexpolyline(inp: str) -> list[tuple[float, float]]:
    """
    Convert a flexpolyline of lat-long coordinates to a list of (lng, lat) tuples.

    We convert from geographic coordinates (lat, lng) to lng, lat
    because geopandas likes it that way, see
    https://geopandas.org/en/stable/gallery/create_geopandas_from_pandas.html.
    """
    inp = flexpolyline.decode(inp)
    return [(float(lng), float(lat)) for (lat, lng) in inp]


def point_to_string(p: Point) -> str:
    """
    Return a lat-lng string representation of a lng-lat Point object.

    The API expects coordinates in the geographic format.
    """
    return f"{p.y},{p.x}"


def string_to_point(data: str) -> Point:
    try:
        lng, lat = (float(val) for val in data.split(","))
    except Exception:
        raise CasaGeoError(
            f"Bad coordinate string {data!r} a good one looks like this 0.0,0.0"
        )
    return Point(lng, lat)

def implicit_enum_parameters(func):
    """
    Decorator that allows parameters of enum type to accept values of
    their underlying type directly.
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        signature = inspect.signature(func)
        bound_args = signature.bind(*args, **kwargs)
        for k, v in bound_args.arguments.items():
            param = signature.parameters[k]
            if (
                    param != inspect.Parameter.empty
                    and isinstance(param.annotation, type)
                    and issubclass(param.annotation, Enum)
            ):
                bound_args.arguments[k] = param.annotation(v)

        return func(*bound_args.args, **bound_args.kwargs)

    return wrapper


def duplicates(inp: list):
    ctr = Counter(inp)
    return [v for v, c in ctr.items() if c > 1]


def cli_catch_exceptions(main) -> int:
    try:
        main()
    except Exception as e:
        # this is not pretty, but it will look pretty
        print(f"{e.__class__.__name__}: {e}", file=sys.stderr)
        return 1
    return 0


def cli_throwing_main(
        parser: argparse.ArgumentParser,
        main: Callable[[argparse.Namespace], dict | pd.DataFrame]
):
    args = parser.parse_args()
    if not args.key:
        raise CasaGeoError(
            "Please specify a key in the environment variable CASAGEO_KEY or the --key argument"
        )

    f = sys.stdout
    if args.output:
        f = open(args.output, "w", encoding="utf-8")

    result = main(args)

    if type(result) is dict:
        json.dump(result, f, indent=args.indent, ensure_ascii=False)
    elif type(result) is pd.DataFrame:
        result.to_csv(f, sep=";")


def cli_main(
        parser: argparse.ArgumentParser,
        main: Callable[[argparse.Namespace], dict | pd.DataFrame]
):
    exit(cli_catch_exceptions(lambda: cli_throwing_main(parser, main)))



TABLE_ORDER = [
    "id",
    "subid",
    "inputStr",
    "label",
    "street",
    "houseNumber",
    "postalCode",
    "city",
    "county",
    "state",
    "countryName",
    "access_lat",
    "access_lng",
    "pos_lat",
    "pos_lng",
    "resultType",
    "houseNumberType",
    "countryCode",
    "stateCode",
    "countyCode",
    "scoreQuery",
    "score_country",
    "score_city",
    "score_houseNumber",
    "score_postalCode",
    "score_streets",
]

ALLOWED_COLUMNS = {
    "id",
    "street",
    "housenumber",
    "postalcode",
    "city",
    "ctrycode",
    "address_string",
    "_",
}

GEOCODE_COST = 3

SERVER = "https://cg-license.casageo.eu/"
# SERVER = "https://cgl-server02.de/"



"""Casageocoder is responsible for encoding and decoding of addresses."""
import argparse

import casageo.client
import pandas as pd
from casageo.client.api.v1 import v1_geocoder_create

import casageo._parsers as parsers
import casageo._util as util
import casageo.account as account
import casageo.consts
from casageo._util import load_json
from casageo.types import CasaGeoError


def _get_dict(d: dict, loc: str):
    for key in loc.split("."):
        if not d or key not in d:
            return None
        d = d[key]
    return d


def _addr_fields_to_str(
        *,
        country: str = "",
        state: str = "",
        county: str = "",
        city: str = "",
        district: str = "",
        street: str = "",
        housenumber: str = "",
        postalcode: str = "",
        address_string: str = "",
        id: str = "",  # this is not relevant for the address but is needed to make the bulk geocoder happy
) -> str:
    if address_string:
        return address_string
    return f"{street} {housenumber}, {district}, {postalcode} {city}, {county}, {state}, {country}"


def _ensure_float(n: int | float | None) -> float | None:
    if isinstance(n, int):
        return float(n)
    return n


def _get_first(n: list | None):
    if isinstance(n, list):
        return n[0]
    return n


def _dict2df(data: dict):
    """
    Data is a dict of the form `{ index1 : { index2 : here_response }}`.
    """
    out = []
    for i1, v1 in data.items():
        for i2, v in enumerate(v1["items"]):
            out.append(
                {
                    "id": i1,
                    "subid": i2,
                    "inputStr": v1.get("inputStr"),
                    "label": _get_dict(v, "address.label"),
                    "street": _get_dict(v, "address.street"),
                    "houseNumber": _get_dict(v, "address.houseNumber"),
                    "postalCode": _get_dict(v, "address.postalCode"),
                    "city": _get_dict(v, "address.city"),
                    "county": _get_dict(v, "address.county"),
                    "state": _get_dict(v, "address.state"),
                    "stateCode": _get_dict(v, "address.stateCode"),
                    "countryName": _get_dict(v, "address.countryName"),
                    "access_lat": _ensure_float(
                        _get_dict(_get_first(v.get("access")), "lat")
                    ),
                    "access_lng": _ensure_float(
                        _get_dict(_get_first(v.get("access")), "lng")
                    ),
                    "pos_lat": _ensure_float(_get_dict(v, "position.lat")),
                    "pos_lng": _ensure_float(_get_dict(v, "position.lng")),
                    "resultType": v.get("resultType"),
                    "houseNumberType": v.get("houseNumberType"),
                    "countryCode": _get_dict(v, "address.countryCode"),
                    "countyCode": _get_dict(v, "address.countyCode"),
                    "scoreQuery": _ensure_float(_get_dict(v, "scoring.queryScore")),
                    "score_country": _ensure_float(
                        _get_dict(v, "scoring.fieldScore.country")
                    ),
                    "score_city": _ensure_float(
                        _get_dict(v, "scoring.fieldScore.city")
                    ),
                    "score_houseNumber": _ensure_float(
                        _get_dict(v, "scoring.fieldScore.houseNumber")
                    ),
                    "score_postalCode": _ensure_float(
                        _get_dict(v, "scoring.fieldScore.postalCode")
                    ),
                    "score_streets": _ensure_float(
                        _get_first(_get_dict(v, "scoring.fieldScore.streets"))
                    ),
                    # "score_streets": Maybe(v)["scoring"]["fieldScore"]["streets"][0].map(float).get(),
                }
            )
    df = pd.DataFrame(out)
    df = df.reindex(columns=casageo.consts.TABLE_ORDER)
    return df


class CasaGeoCoder:
    def __init__(self, key: str, lang: str | None = None) -> None:
        self.lang = lang
        self.client = casageo.client.AuthenticatedClient(
            base_url=casageo.consts.SERVER, token=key, prefix=""
        )

    def _request_str_json(self, addr: str) -> dict:
        response = load_json(v1_geocoder_create.sync_detailed(client=self.client, q=addr))
        response["inputStr"] = addr
        return response

    def address_json(self, address: str | dict) -> dict:
        """
        Geocode an address and return the API response as a dict.

        For the result structure see
        https://www.here.com/docs/bundle/geocoding-and-search-api-developer-guide/page/topics-api/code-geocode-address.html.
        """
        if isinstance(address, dict):
            # TODO: This should use a qualified query instead.
            address = _addr_fields_to_str(**address)
        return self._request_str_json(address)

    def address(self, address: str | dict) -> pd.DataFrame:
        """Geocode an address and return the API response as a dataframe."""
        return _dict2df({"command": self.address_json(address)})

    def bulk_address_json(self, address_df: pd.DataFrame) -> dict:
        """
        Geocode all addresses in the input dataframe and return a dict.

        Warning: If a request errors it may drop the previous data.
        """
        if account.account_info(self.client.token)["credits"] < len(address_df) * casageo.consts.GEOCODE_COST:
            raise CasaGeoError(
                f"Insufficient credits to geocode {len(address_df)} addresses"
            )
        out = {}
        for i, row in address_df.iterrows():
            out[i] = self.address_json(row.to_dict())
            out[i]["id"] = row["id"]
        return out

    def bulk_address(self, address_df: pd.DataFrame) -> pd.DataFrame:
        """
        Geocode all addresses in the input dataframe and return a dataframe.

        Warning: If a request errors it may drop the previous data.
        """
        return _dict2df(self.bulk_address_json(address_df))


def _main(args: argparse.Namespace) -> dict | pd.DataFrame:
    cgc = CasaGeoCoder(args.key, lang=args.lang)

    if args.get_info:
        return account.account_info(args.key)
    elif args.addrstr:
        if args.json:
            return cgc.address_json(args.addrstr)
        else:
            return cgc.address(args.addrstr)
    elif args.file:
        df = pd.read_csv(args.file, sep=";")
        if args.format:
            column_format = args.format.split(",")
            if len(column_format) != len(df.columns):
                raise CasaGeoError(
                    "The amount of columns in format and data must be equal"
                )

            if not set(column_format).issubset(casageo.consts.ALLOWED_COLUMNS):
                raise CasaGeoError(
                    f"The format must only contain {', '.join(casageo.consts.ALLOWED_COLUMNS)}."
                )

            dupes = util.duplicates(column_format)
            while "_" in dupes:
                dupes.remove("_")
            if dupes:
                raise CasaGeoError(
                    "There cannot be multiple columns of the same name except for _"
                )

            # if there is more than one _ rename them to _1, _2 etc
            temp_format = []
            for i, name in enumerate(column_format):
                if name == "_":
                    name += str(i)
                temp_format.append(name)
            column_format = temp_format

            df.columns = column_format

        if 'id' not in df.columns:
            raise CasaGeoError("DataFrame must contain an 'id' column")

        if df['id'].duplicated().any():
            raise CasaGeoError("'id' column contains duplicate values")

        if args.json:
            return cgc.bulk_address_json(df)
        else:
            return cgc.bulk_address(df)
    else:
        raise CasaGeoError("Please provide an argument")


if __name__ == "__main__":
    util.cli_main(parsers.get_coder_parser(), _main)



import argparse

import casageo.client
import casageo.client.types
import geopandas as gpd
import pandas as pd
from casageo.client.api.v1 import (
    v1_isolines_create,
    v1_poi_retrieve,
    v1_revgeocode_retrieve,
    v1_routes_create,
)
from casageo.client.models.v1_isolines_create_rangetype import V1IsolinesCreateRangetype
from casageo.client.models.v1_isolines_create_routing_mode import (
    V1IsolinesCreateRoutingMode,
)
from casageo.client.models.v1_isolines_create_transport_mode import (
    V1IsolinesCreateTransportMode,
)
from casageo.client.models.v1_routes_create_routing_mode import (
    V1RoutesCreateRoutingMode,
)
from casageo.client.models.v1_routes_create_transport_mode import (
    V1RoutesCreateTransportMode,
)
from shapely.geometry import MultiLineString, MultiPolygon, Point, Polygon

import casageo._parsers as parsers
import casageo._util as util
import casageo.account
import casageo.consts
from casageo._util import load_json, implicit_enum_parameters, point_to_string
from casageo.types import CasaGeoError


class CasaGeoSpatial:
    def __init__(self, key: str, lang: str | None = None):
        self.lang = lang
        self.client = casageo.client.AuthenticatedClient(
            casageo.consts.SERVER, key, prefix=""
        )

    @implicit_enum_parameters
    def isolines_json(
            self,
            *,
            range_values: list[float],
            point: Point,
            range_type: V1IsolinesCreateRangetype = V1IsolinesCreateRangetype.TIME,
            transport_mode: V1IsolinesCreateTransportMode = V1IsolinesCreateTransportMode.CAR,
            routing_mode: V1IsolinesCreateRoutingMode = V1IsolinesCreateRoutingMode.FAST,
    ) -> dict:
        """
        Response Structure: https://www.here.com/docs/bundle/isoline-routing-api-developer-guide-v8/page/topics/isoline-response.html
        """

        if range_type == V1IsolinesCreateRangetype.TIME:
            range_values = map(lambda x: x * 60, range_values) # We want minutes
        range_values = map(int, range_values)

        response = v1_isolines_create.sync_detailed(
            client=self.client,
            origin=util.point_to_string(point),
            routing_mode=routing_mode,
            transport_mode=transport_mode,
            rangetype=range_type,
            rangevalues=",".join(map(str, range_values)),
        )
        data = load_json(
            response
        )

        for isoline in data["isolines"]:
            for polygon in isoline["polygons"]:
                polygon["outer"] = util.load_flexpolyline(polygon["outer"])

        return data

    def isolines(
            self,
            *,
            range_values: list[float],
            point: Point,
            range_type: V1IsolinesCreateRangetype = V1IsolinesCreateRangetype.TIME,
            transport_mode: V1IsolinesCreateTransportMode = V1IsolinesCreateTransportMode.CAR,
            routing_mode: V1IsolinesCreateRoutingMode = V1IsolinesCreateRoutingMode.FAST,
    ) -> gpd.GeoDataFrame:


        data = self.isolines_json(
            range_values=range_values,
            point=point,
            range_type=range_type,
            transport_mode=transport_mode,
            routing_mode=routing_mode,
        )
        return gpd.GeoDataFrame(
            {
                "geometry": [
                    MultiPolygon(
                        Polygon(polygon["outer"])
                        for polygon in isoline["polygons"]
                    )
                    for isoline in data["isolines"]
                ],
                "names": [
                    f"{isoline['range']['type']}: {isoline['range']['value']}"
                    for isoline in data["isolines"]
                ],
            }, crs="EPSG:4326"
        )

    @implicit_enum_parameters
    def routes_json(
            self,
            *,
            return_: str,
            a: Point,
            b: Point,
            routing_mode: V1RoutesCreateRoutingMode,
            transport_mode: V1RoutesCreateTransportMode,
    ) -> dict:
        data = load_json(
            v1_routes_create.sync_detailed(
                return_=return_,
                client=self.client,
                origin=point_to_string(a),
                destination=point_to_string(b),
                routing_mode=routing_mode,
                transport_mode=transport_mode,
            )
        )

        for route in data["routes"]:
            for section in route["sections"]:
                if "polyline" in section:
                    section["polyline"] = util.load_flexpolyline(
                        section["polyline"]
                    )

        return data

    def routes(
            self,
            *,
            return_: str,
            a: Point,
            b: Point,
            routing_mode: V1RoutesCreateRoutingMode,
            transport_mode: V1RoutesCreateTransportMode,
    ) -> gpd.GeoDataFrame:
        data = self.routes_json(
            return_=return_,
            a=a,
            b=b,
            routing_mode=routing_mode,
            transport_mode=transport_mode,
        )
        geometry = [
            MultiLineString(
                [
                    section["polyline"]
                    for section in route["sections"]
                    if "polyline" in section
                ]
            )
            for route in data["routes"]
        ]

        return gpd.GeoDataFrame(geometry=geometry, crs="EPSG:4326")

    def poi_json(
            self,
            point: Point,
            limit: int,
    ) -> dict:
        return load_json(
            v1_poi_retrieve.sync_detailed(
                client=self.client,
                at=point_to_string(point),
                limit=limit,
            )
        )

    def poi(
            self,
            point: Point,
            limit: int,
    ) -> gpd.GeoDataFrame:
        data = self.poi_json(point, limit)
        return gpd.GeoDataFrame(
            {
                "geometry": [
                    Point(item["position"]["lng"], item["position"]["lat"])
                    for item in data["items"]
                ],
                "names": [item["title"] for item in data["items"]],
            }
            , crs="EPSG:4326")

    def revgeocode_json(self, point: Point) -> dict:
        return load_json(
            v1_revgeocode_retrieve.sync_detailed(
                client=self.client,
                at=point_to_string(point),
                lang=(self.lang if self.lang else casageo.client.types.UNSET),
            )
        )

    def revgeocode_str(self, point: Point) -> str:
        return self.revgeocode_json(point)["items"][0]["title"]

    def revgeocode(self, point: Point) -> gpd.GeoDataFrame:
        data = self.revgeocode_json(point)
        return gpd.GeoDataFrame(
            {
                "geometry": [
                    Point(item["position"]["lng"], item["position"]["lat"])
                    for item in data["items"]
                ],
                "names": [item["title"] for item in data["items"]],
            }, crs="EPSG:4326"
        )


def _main(args: argparse.Namespace) -> dict | pd.DataFrame:
    cgs = CasaGeoSpatial(args.key, args.lang)
    match args.command:
        case "isolines":
            p = util.string_to_point(args.point)
            range_values = list(map(float, args.range.split(",")))

            if args.raw:
                return cgs.isolines_json(
                    range_values=range_values,
                    point=p,
                    range_type=V1IsolinesCreateRangetype(args.rangetype),
                    transport_mode=V1IsolinesCreateTransportMode(args.vehicle),
                    routing_mode=V1IsolinesCreateRoutingMode(args.routing_mode),
                )
            else:
                return cgs.isolines(
                    range_values=range_values,
                    point=p,
                    range_type=V1IsolinesCreateRangetype(args.rangetype),
                    transport_mode=V1IsolinesCreateTransportMode(args.vehicle),
                    routing_mode=V1IsolinesCreateRoutingMode(args.routing_mode),
                )
        case "routes":
            a = util.string_to_point(args.pointa)
            b = util.string_to_point(args.pointb)
            if args.raw:
                return cgs.routes_json(
                    return_=vars(args)["return"],  # return is a keyword we can't do args.return
                    a=a,
                    b=b,
                    routing_mode=V1RoutesCreateRoutingMode(args.routing_mode),
                    transport_mode=V1RoutesCreateTransportMode(args.transport_mode),
                )
            else:
                return cgs.routes(
                    return_=vars(args)["return"],
                    a=a,
                    b=b,
                    routing_mode=V1RoutesCreateRoutingMode(args.routing_mode),
                    transport_mode=V1RoutesCreateTransportMode(args.transport_mode),
                )
        case "poi":
            p = util.string_to_point(args.point)
            if args.raw:
                return cgs.poi_json(p, limit=args.limit)
            else:
                return cgs.poi(
                    p, limit=args.limit
                )
        case "revgeocode":
            p = util.string_to_point(args.at)
            if args.raw:
                return cgs.revgeocode_json(p)
            else:
                return cgs.revgeocode(p)
        case "get-info":
            return casageo.account.account_info(args.key)
        case _:
            raise CasaGeoError(f"Unknown command: {args.command}")


if __name__ == "__main__":
    util.cli_main(parsers.get_spatial_parser(), _main)



import argparse
import os
from datetime import datetime


# This has to be in a separate file because the manpage generator needs it.

def get_spatial_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser("casageospatial")
    parser.add_argument(
        "--output",
        help="File to output to if not given will default to stdout.",
        default=None,
    )
    parser.add_argument(
        "--indent",
        help="The indent of the output json if not given wont be indented",
        default=None,
        type=int,
    )
    parser.add_argument(
        "--file", type=str, default="", help="Provide a csv seperated by ;"
    )
    parser.add_argument(
        "--key",
        help="Provide a casageo spatial apikey.",
        default=os.getenv("CASAGEO_KEY"),
    )
    parser.add_argument(
        "--lang", "-l", default="en_US", help="Output langauge for the data."
    )

    subparsers = parser.add_subparsers(dest="command", required=True)
    routes_parser = subparsers.add_parser("routes")
    routes_parser.add_argument(
        "--raw",
        action="store_true",
        help="Output the raw json from the api instead of a geojson.",
    )

    routes_parser.add_argument(
        "--routing-mode",
        "-r",
        choices=["fast", "short"],
        default="fast",
    )
    routes_parser.add_argument(
        "--transport-mode",
        "-t",
        choices=[
            "bicycle",
            "bus",
            "car",
            "pedestrian",
            "privateBus",
            "scooter",
            "taxi",
            "truck",
        ],
        default="car",
    )
    routes_parser.add_argument("return")
    routes_parser.add_argument("pointa")
    routes_parser.add_argument("pointb")

    isolines_parser = subparsers.add_parser("isolines")

    isolines_parser.add_argument(
        "--routing-mode",
        "-r",
        choices=["fast", "short"],
        default="fast",
    )
    isolines_parser.add_argument(
        "--vehicle",
        "-v",
        choices=[
            "bicycle",
            "bus",
            "car",
            "pedestrian",
            "privateBus",
            "scooter",
            "taxi",
            "truck",
        ],
        default="car",
    )
    isolines_parser.add_argument("--avoid", "-a", type=str)
    isolines_parser.add_argument(
        "--time",
        type=datetime.fromisoformat,
        help="Provide a date and time in ISO format.",
    )

    isolines_parser.add_argument(
        "--raw",
        action="store_true",
        help="Outputs the raw api response with minimal modifications.",
    )
    isolines_parser.add_argument(
        "--rangetype", choices=["consumption", "distance", "time"], default="time"
    )
    isolines_parser.add_argument("point")
    isolines_parser.add_argument(
        "ranges",
        help="Provide comma seperated ranges in either meters or minutes depending on --range-type.",
    )

    poi_parser = subparsers.add_parser("poi")

    poi_parser.add_argument("--limit", "-l", type=int, default=10)
    poi_parser.add_argument(
        "--categories",
        "-c",
        type=str,
        default="100,200,300,400,500,600,700,800,900",
        help="https://www.here.com/docs/bundle/geocoding-and-search-api-developer-guide/page/topics-places/places-category-system-full.html",
    )
    poi_parser.add_argument("point", type=str)
    poi_parser.add_argument("--raw", action="store_true", help="Output raw json")

    revgeocode_parser = subparsers.add_parser("revgeocode")
    revgeocode_parser.add_argument("at", type=str, help="Specify a location.")
    revgeocode_parser.add_argument("--raw", action="store_true", help="Output raw json")

    subparsers.add_parser("get-info")
    return parser


def get_coder_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser("casageocoder")
    parser.add_argument(
        "--output",
        help="File to output to if not given will default to stdout.",
        default=None,
    )
    parser.add_argument(
        "--indent",
        help="The indent of the output json if not given wont be indented",
        default=None,
        type=int,
    )
    parser.add_argument(
        "--file", type=str, default="", help="Provide a csv seperated by ;"
    )
    parser.add_argument(
        "--format",
        type=str,
        default="",
        help="Provide a file format, the format is like a csv header, but there are only a few valid column names.",
    )
    parser.add_argument("--street", type=str, default="", help="Provide the street")
    parser.add_argument(
        "--housenumber", type=str, default="", help="Provide the house number"
    )
    parser.add_argument(
        "--postalcode", type=str, default="", help="Provide the postal code"
    )
    parser.add_argument("--city", type=str, default="", help="Provide the city")
    parser.add_argument(
        "--ctrycode", type=str, default="", help="Provide the country code"
    )
    parser.add_argument(
        "--get-info",
        action="store_true",
        help="Prints info about your license key",
    )
    parser.add_argument(
        "--key",
        type=str,
        help="Provide the API key in an argument instead of a file",
        default=os.getenv("CASAGEO_KEY"),
    )
    parser.add_argument(
        "--addrstr", type=str, default="", help="Provide the entire query as one string"
    )
    parser.add_argument("--lang", type=str, default=None, help="Provide a language")
    parser.add_argument(
        "--json", action="store_true", help="output as a json instead of a csv"
    )

    return parser



from casageo.client import AuthenticatedClient
from casageo.client.api.v1 import v1_accountinfo_retrieve

from casageo._util import load_json
from casageo.consts import SERVER


def account_info(key: str) -> dict:
    """Returns a dictionary containing account information.

    - "credits": Remaining credits (int).
    - "app_identifier": Application identifier (str).
    - "expires": Expiry date (str).
    - "expires_unix": Expiry date timestamp (float).
    - "key": License key (str).
    - "user": Username (str).
    """
    return load_json(
        v1_accountinfo_retrieve.sync_detailed(
            client=AuthenticatedClient(base_url=SERVER, token=key, prefix="")
        )
    )
